 /* -*- Mode: c -*-

     Lexer for a minimal programming language "nanoLang"

     Copyright 2015 by Stephan Schulz, schulz@eprover.org.

     This code is released under the GNU General Public Licence
     Version 2.
 */

%option noyywrap
%option yylineno

%{
   #include <string.h>
   #include "ast.h"
   #include "nanoparse.tab.h"   

   int ret_tok(int tok)
   {
      return tok;
   }

   int yycol = 0;

#define YY_USER_ACTION {yylloc.first_line = yylineno; yylloc.first_column = yycol; yycol += yyleng;}

%}

DIGIT      [0-9]
ALPHA      [a-zA-Z]
ALNUM      {ALPHA}|{DIGIT}

INT        {DIGIT}+
ID         {ALPHA}{ALNUM}*


%%

"("  {yylval = ASTAlloc2(t_dummy, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(OPENPAR);}
")"  {yylval = ASTAlloc2(t_dummy, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(CLOSEPAR);}
"*"  {yylval = ASTAlloc2(t_MULT, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(MULT);}
"/"  {yylval = ASTAlloc2(t_DIV, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(DIV);}
"+"  {yylval = ASTAlloc2(t_PLUS, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(PLUS);}
"-"  {yylval = ASTAlloc2(t_MINUS, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(MINUS);}
"="  {yylval = ASTAlloc2(t_EQ, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(EQ);}
"!=" {yylval = ASTAlloc2(t_NEQ, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(NEQ);}
">"  {yylval = ASTAlloc2(t_GT, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(GT);}
"<"  {yylval = ASTAlloc2(t_LT, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(LT);}
">=" {yylval = ASTAlloc2(t_GEQ, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(GEQ);}
"<=" {yylval = ASTAlloc2(t_LEQ, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(LEQ);}
"{"  {yylval = ASTAlloc2(t_dummy, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(OPENCURLY);}
"}"  {yylval = ASTAlloc2(t_dummy, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(CLOSECURLY);}
";"  {yylval = ASTAlloc2(t_dummy, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(SEMICOLON);}
","  {yylval = ASTAlloc2(t_dummy, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(COMA);}


Integer {yylval = ASTAlloc2(t_INTEGER, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
   return ret_tok(INTEGER);}
String  {yylval = ASTAlloc2(t_STRING, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
   return ret_tok(STRING);}
if      {yylval = ASTAlloc2(t_IF, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
   return ret_tok(IF);}
while   {yylval = ASTAlloc2(t_WHILE, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
   return ret_tok(WHILE);}
return  {yylval = ASTAlloc2(t_RETURN, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
   return ret_tok(RETURN);}
print   {yylval = ASTAlloc2(t_PRINT, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
   return ret_tok(PRINT);}
       
{ALPHA}{ALNUM}* {yylval = ASTAlloc2(t_IDENT, yylineno, yycol-yyleng, 
                                   strdup(yytext), 0, NULL, NULL);
                 return ret_tok(IDENT);}

\"[^"]*\"       {yylval = ASTAlloc2(t_STRINGLIT, yylineno, yycol-yyleng, 
                                   strdup(yytext), 0, NULL, NULL);
                 return ret_tok(STRINGLIT);}

{DIGIT}+        {yylval = ASTAlloc2(t_INTLIT, yylineno, yycol-yyleng, 
                                   strdup(yytext), atoll(yytext), NULL, NULL);
                 return ret_tok(INTLIT);}


#[^\n]*\n { /* Eat comments */}

[\n]    {yycol = 0;}

[\r\t ] { /* Skip  whitespace*/ }


 /* Everything else is an invalid character. */
.   {return ret_tok(ERROR);}

%%

